@@ -128,7 +128,7 @@ static GnomeUIInfo menu_defaults[] = {
           GNOME_APP_PIXMAP_STOCK, GTK_STOCK_REVERT_TO_SAVED,
           0,  (GdkModifierType) 0, NULL },
 	/* Print */
-        { GNOME_APP_UI_ITEM, N_("_Print"), N_("Print the current file"),
+        { GNOME_APP_UI_ITEM, N_("_Print..."), N_("Print the current file"),
           NULL, NULL, NULL,
           GNOME_APP_PIXMAP_STOCK, GTK_STOCK_PRINT,
           GNOME_KEY_NAME_PRINT,  GNOME_KEY_MOD_PRINT, NULL },
@@ -144,15 +144,15 @@ static GnomeUIInfo menu_defaults[] = {
           GNOME_APP_PIXMAP_STOCK, GTK_STOCK_CLOSE,
           GNOME_KEY_NAME_CLOSE, GNOME_KEY_MOD_CLOSE, NULL },
 	/* Exit */
-        { GNOME_APP_UI_ITEM, N_("E_xit"), N_("Exit the program"),
+        { GNOME_APP_UI_ITEM, N_("_Quit"), N_("Quit the application"),
           NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-	  GTK_STOCK_QUIT, GNOME_KEY_NAME_EXIT, GNOME_KEY_MOD_EXIT,
+	  GTK_STOCK_QUIT, GNOME_KEY_NAME_QUIT, GNOME_KEY_MOD_QUIT,
 	    NULL },
 	/*
 	 * The "Edit" menu
 	 */
 	/* Cut */
-        { GNOME_APP_UI_ITEM, N_("C_ut"), N_("Cut the selection"),
+        { GNOME_APP_UI_ITEM, N_("Cu_t"), N_("Cut the selection"),
           NULL, NULL, NULL,
           GNOME_APP_PIXMAP_STOCK, GTK_STOCK_CUT,
           GNOME_KEY_NAME_CUT, GNOME_KEY_MOD_CUT, NULL },
@@ -188,13 +188,13 @@ static GnomeUIInfo menu_defaults[] = {
           GNOME_APP_PIXMAP_STOCK, GTK_STOCK_FIND,
           GNOME_KEY_NAME_FIND, GNOME_KEY_MOD_FIND, NULL },
 	/* Find Again */
-        { GNOME_APP_UI_ITEM, N_("Find _Again"),
+        { GNOME_APP_UI_ITEM, N_("Find Ne_xt"),
           N_("Search again for the same string"),
           NULL, NULL, NULL,
           GNOME_APP_PIXMAP_STOCK, GTK_STOCK_FIND,
           GNOME_KEY_NAME_FIND_AGAIN, GNOME_KEY_MOD_FIND_AGAIN, NULL },
 	/* Replace */
-        { GNOME_APP_UI_ITEM, N_("_Replace..."), N_("Replace a string"),
+        { GNOME_APP_UI_ITEM, N_("R_eplace..."), N_("Replace a string"),
           NULL, NULL, NULL,
           GNOME_APP_PIXMAP_STOCK, GTK_STOCK_FIND_AND_REPLACE,
           GNOME_KEY_NAME_REPLACE, GNOME_KEY_MOD_REPLACE, NULL },
@@ -208,7 +208,7 @@ static GnomeUIInfo menu_defaults[] = {
 	 * The Settings menu
 	 */
 	/* Settings */
-        { GNOME_APP_UI_ITEM, N_("_Preferences..."),
+        { GNOME_APP_UI_ITEM, N_("Prefere_nces..."),
           N_("Configure the application"),
           NULL, NULL,
 	  NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_PREFERENCES,
@@ -222,7 +222,7 @@ static GnomeUIInfo menu_defaults[] = {
           N_("About this application"), NULL, NULL,
 	  NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_ABOUT,
           0,  (GdkModifierType) 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Select All"),
+	{ GNOME_APP_UI_ITEM, N_("Select _All"),
           N_("Select everything"),
           NULL, NULL, NULL,
           GNOME_APP_PIXMAP_NONE, NULL,
@@ -299,7 +299,7 @@ static gchar *menu_names[] =
   "print",
   "print-setup",
   "close",
-  "exit",
+  "quit",
   "cut",
   /* 10 */
   "copy",
@@ -329,6 +329,37 @@ static gchar *menu_names[] =
 };
 
 
+static GtkWidget *
+scale_pixbuf (GdkPixbuf *pixbuf, GtkIconSize icon_size) {
+	double pix_x, pix_y;
+	int width, height;
+
+	gtk_icon_size_lookup (icon_size, &width, &height);
+
+        pix_x = gdk_pixbuf_get_width (pixbuf);
+        pix_y = gdk_pixbuf_get_height (pixbuf);
+
+	/* Only scale if the image doesn't match the required
+	 * icon size 
+	 */
+        if (pix_x > width || pix_y > height) {
+        	double greatest;
+        	GdkPixbuf *scaled;
+		GtkWidget *image;
+
+                greatest = pix_x > pix_y ? pix_x : pix_y;
+                scaled = gdk_pixbuf_scale_simple (pixbuf,
+                                                  (width/ greatest) * pix_x,
+                                                  (height/ greatest) * pix_y,
+                                                   GDK_INTERP_BILINEAR);
+                image = gtk_image_new_from_pixbuf (scaled);
+                g_object_unref (G_OBJECT (scaled));
+		return image;
+	}
+	
+	return gtk_image_new_from_pixbuf (pixbuf);
+}
+
 /* Creates a pixmap appropriate for items.  */
 
 static GtkWidget *
@@ -351,7 +382,7 @@ create_pixmap (GnomeUIPixmapType pixmap_type, gconstpointer pixmap_info,
 			GdkPixbuf *pixbuf = gdk_pixbuf_new_from_xpm_data
 				((const char **)pixmap_info);
 			if (pixbuf != NULL) {
-				pixmap = gtk_image_new_from_pixbuf (pixbuf);
+				pixmap = scale_pixbuf (pixbuf, icon_size);
 				g_object_unref (G_OBJECT (pixbuf));
 			}
 		}
@@ -370,7 +401,10 @@ create_pixmap (GnomeUIPixmapType pixmap_type, gconstpointer pixmap_info,
 			g_warning ("Could not find GNOME pixmap file %s",
 					(char *) pixmap_info);
 		else {
-			pixmap = gtk_image_new_from_file (name);
+			GdkPixbuf *pixbuf;
+			pixbuf = gdk_pixbuf_new_from_file (name, NULL);
+			pixmap = scale_pixbuf (pixbuf, icon_size);
+			g_object_unref (G_OBJECT (pixbuf));
 			g_free (name);
 		}
 
@@ -587,8 +621,8 @@ install_menuitem_hint_to_statusbar(GnomeUIInfo* uiinfo, GtkStatusbar* bar)
 
 /**
  * gnome_app_install_statusbar_menu_hints
- * @bar: Pointer to Gtk+ status bar object
- * @uiinfo: Gnome UI info for the menu to be changed
+ * @bar: Pointer to #GtkStatusbar instance.
+ * @uiinfo: #GnomeUIInfo for the menu to be changed.
  *
  * Description:
  * Install menu hints for the given status bar.
@@ -617,6 +651,7 @@ gnome_app_install_statusbar_menu_hints (GtkStatusbar* bar,
 		case GNOME_APP_UI_ITEM:
 		case GNOME_APP_UI_TOGGLEITEM:
 		case GNOME_APP_UI_SEPARATOR:
+		case GNOME_APP_UI_HELP:
 			install_menuitem_hint_to_statusbar(uiinfo, bar);
 			break;
 		case GNOME_APP_UI_RADIOITEMS:
@@ -702,6 +737,12 @@ install_menuitem_hint_to_appbar(GnomeUIInfo* uiinfo, GnomeAppBar* bar)
  * Configure all user-configurable elements in the given UI info
  * structure.  This includes loading and setting previously-set options from
  * GNOME config files.
+ *
+ * Normally, gnome_app_create_menus() calls this function for the developer,
+ * but if something needs to be altered afterwards, this function can be called
+ * first. The main reason for this function being a public interface is so that
+ * it can be called from gnome_popup_menu_new(), which clears a copy of the
+ * pass in #GnomeUIInfo structures.
  */
 
 void
@@ -782,11 +823,13 @@ gnome_app_ui_configure_configurable (GnomeUIInfo* uiinfo)
 
 /**
  * gnome_app_install_appbar_menu_hints
- * @appbar: Pointer to GNOME app bar object.
- * @uiinfo: GNOME UI info for menu
+ * @appbar: An existing #GnomeAppBar instance.
+ * @uiinfo: A #GnomeUIInfo array of a menu for which hints will be set.
  *
  * Description:
- * Install menu hints for the given GNOME app bar object.
+ * Install menu hints for the given @appbar object. This function cannot just
+ * be called automatically, since it is impossible to reliably find the correct
+ * @appbar.
  */
 
 void
@@ -818,6 +861,7 @@ gnome_app_install_appbar_menu_hints (GnomeAppBar* appbar,
 		case GNOME_APP_UI_ITEM:
 		case GNOME_APP_UI_TOGGLEITEM:
 		case GNOME_APP_UI_SEPARATOR:
+		case GNOME_APP_UI_HELP:
 			install_menuitem_hint_to_appbar(uiinfo, appbar);
 			break;
 		case GNOME_APP_UI_RADIOITEMS:
@@ -835,11 +879,11 @@ gnome_app_install_appbar_menu_hints (GnomeAppBar* appbar,
 
 /**
  * gnome_app_install_menu_hints
- * @app: Pointer to GNOME app object
- * @uiinfo: GNOME UI menu for which hints will be set
+ * @app: An existing #GnomeAppBar instance.
+ * @uiinfo: A #GnomeUIInfo array of a menu for which hints will be set.
  *
  * Description:
- * Set menu hints for the GNOME app object's attached status bar.
+ * Set menu hints for the @app object's attached status bar.
  */
 
 void
@@ -870,6 +914,12 @@ gnome_save_accels (gpointer data)
 	return TRUE;
 }
 
+/**
+ * gnome_accelerators_sync:
+ *
+ * Flush the accelerator definitions into the application specific
+ * configuration file $HOME/.gnome2/accels/<app-id>.
+ */
 void
 gnome_accelerators_sync (void)
 {
@@ -1073,145 +1123,14 @@ create_radio_menu_items (GtkMenuShell *menu_shell, GnomeUIInfo *uiinfo,
 	return pos;
 }
 
-static char *
-quote_string (const char *string)
-{
-	GString *new_string = g_string_new ("'");
-	int i;
-
-	for (i = 0; string[i] != '\0'; i++) {
-		if (string[i] == '\'') {
-			g_string_append (new_string, "'\\''");
-		} else {
-			g_string_append_c (new_string, string[i]);
-		}
-	}
-	g_string_append_c (new_string, '\'');
-
-	return g_string_free (new_string, FALSE);
-}
-
-static char *
-get_toc_location (const char *locale, const char *docname)
-{
-	char *cmd;
-	char *quoted;
-	char toc_location[PATH_MAX + 1];
-	FILE *fp;
-	int bytes;
-
-	quoted = quote_string (docname);
-	cmd = g_strdup_printf ("scrollkeeper-get-toc-from-docpath %s/%s",
-			       locale, quoted);
-	g_free (quoted);
-
-	fp = popen (cmd, "r");
-
-	g_free (cmd);
-
-	bytes = fread ((void *)toc_location, sizeof(char), PATH_MAX, fp);
-	if (pclose (fp) == 0 &&
-	    bytes > 0) {
-		/* cut the trailing '\n', and make
-		 * sure there is a null char */
-		toc_location[bytes - 1] = '\0';
-		return g_strdup (toc_location);
-	} else {
-		return NULL;
-	}
-}
-
-static char *
-get_best_toc_location (const char *docname)
-{
-	const GList *list = gnome_i18n_get_language_list ("LC_MESSAGES");
-
-	while (list != NULL) {
-		const char *locale = list->data;
-		char *ret = get_toc_location (locale, docname);
-		if (ret != NULL)
-			return ret;
-
-		list = list->next;
-	}
-
-	return NULL;
-}
-
-static GSList *
-get_topics (const char *docname)
-{
-	xmlDocPtr toc_doc;
-	GSList *list = NULL;
-	xmlNodePtr next_child;
-	errorSAXFunc xml_error_handler;
-    	warningSAXFunc xml_warning_handler;
-    	fatalErrorSAXFunc xml_fatal_error_handler;
-
-	char *toc_location = get_best_toc_location (docname);
-
-	if (toc_location == NULL)
-		return NULL;
-
-	/* Evil thing stolen from nautilus, this just looks
-	 * so evil it's not even pretty */
-	xml_error_handler = xmlDefaultSAXHandler.error;
-	xmlDefaultSAXHandler.error = NULL;
-	xml_warning_handler = xmlDefaultSAXHandler.warning;
-	xmlDefaultSAXHandler.warning = NULL;
-	xml_fatal_error_handler = xmlDefaultSAXHandler.fatalError;
-	xmlDefaultSAXHandler.fatalError = NULL;
-	toc_doc = xmlParseFile (toc_location);
-	xmlDefaultSAXHandler.error = xml_error_handler;
-    	xmlDefaultSAXHandler.warning = xml_warning_handler;
-    	xmlDefaultSAXHandler.fatalError = xml_fatal_error_handler;
-
-	if (toc_doc == NULL) {
-		g_free (toc_location);
-		return NULL;
-	}
-
-	/* FIXME: this is a TREE!!!! we only read toplevels */
-	for (next_child = toc_doc->xmlRootNode->xmlChildrenNode;
-	     next_child != NULL;
-	     next_child = next_child->next) {
-		if (g_ascii_strncasecmp (next_child->name,
-					 "tocsect", strlen ("tocsect")) == 0) {
-			char *content = xmlNodeGetContent (next_child);
-			char *linkid = xmlGetProp (next_child, "linkid");
-
-			if (content != NULL &&
-			    linkid != NULL) {
-				list = g_slist_append (list,
-						       g_strdup (content));
-				list = g_slist_append (list,
-						       g_strdup (linkid));
-			}
-
-			if (content != NULL)
-				xmlFree (content);
-		}
-	}
-
-	xmlFreeDoc (toc_doc);
-
-	return list;
-}
-
 static void
 help_view_display_callback (GtkWidget *w, gpointer data)
 {
-	char *section = data;
-	char *docname = g_object_get_data (G_OBJECT (w), "docname");
+	char *file_name = data;
 
 	/* FIXME: handle errors somehow */
-	if (docname == NULL ||
-	    section == NULL)
-		return;
-
-	/* FIXME: handle errors somehow */
-	gnome_help_display (docname,
-			    section,
+	gnome_help_display (file_name,
+			    NULL,
 			    NULL /* error */);
 }
 
@@ -1220,52 +1139,38 @@ help_view_display_callback (GtkWidget *w, gpointer data)
 static int
 create_help_entries (GtkMenuShell *menu_shell, GnomeUIInfo *uiinfo, gint pos)
 {
-	GSList *topics, *cur;
+	gchar *path;
 
-	uiinfo->widget = NULL; /* No relevant widget, as we may have created
-				  several of them */
+	uiinfo->widget = gtk_image_menu_item_new_with_mnemonic (_("_Contents"));
+	uiinfo->hint = g_strdup (_("View help for this application"));
 
-	if (!uiinfo->moreinfo) {
-		g_warning ("GnomeUIInfo->moreinfo cannot be NULL for "
-				"GNOME_APP_UI_HELP");
-		return pos;
-	}
+	setup_image_menu_item (uiinfo->widget, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_HELP);
 
-	topics = get_topics ((const char *)uiinfo->moreinfo);
+	g_signal_connect_data (uiinfo->widget, "activate",
+			       G_CALLBACK (help_view_display_callback),
+			       g_strdup (uiinfo->moreinfo), 
+			       (GClosureNotify) g_free, 0);
+	
+	gtk_menu_shell_insert (menu_shell, uiinfo->widget, pos);
 
-	for (cur = topics; cur && cur->next; cur = cur->next->next) {
-		GtkWidget *item;
-		GtkWidget *label;
+	/* Install global accelerator */
 
-		item = gtk_menu_item_new ();
-		label = create_label (cur->data);
-		g_free(cur->data);
-		cur->data = NULL;
+	path = g_strdup_printf("<%s>/help-contents-item", 
+			       gnome_program_get_app_id (gnome_program_get()));
+	
+	/* Associate the key combo with the accel path */
+	gtk_accel_map_add_entry (path, GDK_F1, 0);
 
-		gtk_container_add (GTK_CONTAINER (item), label);
-/*		gtk_widget_lock_accelerators (item); */
+	/* Associate the accel path with the menu item */
+	if (GTK_IS_MENU (menu_shell))
+		gtk_menu_item_set_accel_path (GTK_MENU_ITEM (uiinfo->widget),
+					      path);
 
-		g_object_set_data_full (G_OBJECT (item), "docname",
-					g_strdup (uiinfo->moreinfo),
-					g_free);
+	g_free (path);
+	
+	gtk_widget_show_all (uiinfo->widget);
 
-		g_signal_connect_data (item, "activate",
-				       G_CALLBACK (help_view_display_callback),
-				       cur->next->data,
-				       (GClosureNotify) g_free,
-				       0);
-
-		cur->next->data = NULL;
-
-		gtk_menu_shell_insert (menu_shell, item, pos);
-		pos++;
-
-		gtk_widget_show (item);
-
-	}
-	g_slist_free(topics);
-
-	return pos;
+	return ++pos;
 }
 
 typedef struct
@@ -1317,20 +1222,21 @@ do_ui_signal_connect (GnomeUIInfo        *uiinfo,
 
 /**
  * gnome_app_fill_menu
- * @menu_shell:
- * @uiinfo:
- * @accel_group:
- * @uline_accels:
- * @pos:
+ * @menu_shell: A #GtkMenuShell instance (a menu bar).
+ * @uiinfo: A pointer to the first element in an array of #GnomeUIInfo
+ * structures. The last element of the array should have a type of
+ * #GNOME_APP_UI_ENDOFINFO.
+ * @accel_group: A #GtkAccelGroup.
+ * @uline_accels: %TRUE if underline accelerators will be drawn for the menu
+ * item labels.
+ * @pos: The position in the menu bar at which to start inserting items.
  *
  * Description:
- * Fills the specified menu shell with items created from the specified
- * info, inserting them from the item no. pos on.
- * The accel group will be used as the accel group for all newly created
- * sub menus and serves as the global accel group for all menu item
- * hotkeys. If it is passed as NULL, global hotkeys will be disabled.
- * The uline_accels argument determines whether underline accelerators
- * will be featured from the menu item labels.
+ * Fills the specified @menu_shell with items created from the specified
+ * @uiinfo, inserting them from the item number @pos on.  The @accel_ group
+ * will be used as the accel group for all newly created sub menus and serves
+ * as the global accel group for all menu item hotkeys. If it is passed as
+ * %NULL, global hotkeys will be disabled.
  **/
 
 void
@@ -1362,14 +1268,19 @@ gnome_app_fill_menu (GtkMenuShell  *menu_shell,
 
 /**
  * gnome_app_fill_menu_with_data
- * @menu_shell:
- * @uiinfo:
- * @accel_group:
- * @uline_accels:
- * @pos:
- * @user_data:
+ * @menu_shell: A #GtkMenuShell instance (a menu bar).
+ * @uiinfo: A pointer to the first element in an array of #GnomeUIInfo
+ * structures. The last element of the array should have a type of
+ * #GNOME_APP_UI_ENDOFINFO.
+ * @accel_group: A #GtkAccelGroup.
+ * @uline_accels: %TRUE if underline accelerators will be drawn for the menu
+ * item labels.
+ * @pos: The position in the menu bar at which to start inserting items.
+ * @user_data: Some application-specific data.
  *
  * Description:
+ * This is the same as gnome_app_fill_menu(), except that all the user data
+ * pointers are filled with the value of @user_data.
  **/
 
 void
@@ -1397,25 +1308,70 @@ gnome_app_fill_menu_with_data (GtkMenuShell  *menu_shell,
 				    pos);
 }
 
+static void
+menus_have_tearoff_changed_notify(GConfClient            *client,
+				  guint                   cnxn_id,
+				  GConfEntry             *entry,
+				  gpointer                user_data)
+{
+	GtkWidget *menu;
+	
+	if (entry->value->type != GCONF_VALUE_BOOL)
+		return;
+
+	menu = GTK_WIDGET (user_data);
+	
+	if (gconf_value_get_bool (entry->value)) {
+		GtkWidget *tearoff;
+
+		tearoff = g_object_get_data (G_OBJECT (menu), "gnome-app-tearoff");
+
+		if (tearoff) {
+			/* Do nothing */
+			return;
+		}
+		
+		/* Add the tearoff */
+		tearoff = gtk_tearoff_menu_item_new ();
+		gtk_widget_show (tearoff);
+		g_object_set_data (G_OBJECT (menu), "gnome-app-tearoff", tearoff);
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), tearoff);
+	}
+	else {
+		GtkWidget *tearoff;
+
+		tearoff = g_object_get_data (G_OBJECT (user_data), "gnome-app-tearoff");
+
+		if (!tearoff) {
+			/* Do nothing */
+			return;
+		}
+		
+		/* Remove the tearoff */
+		gtk_widget_destroy (tearoff);
+		g_object_set_data (G_OBJECT (menu), "gnome-app-tearoff", NULL);
+	}
+}
+
 
 /**
  * gnome_app_fill_menu_custom
- * @menu_shell:
- * @uiinfo:
- * @uibdata:
- * @accel_group:
- * @uline_accels:
- * @pos:
+ * @menu_shell: A #GtkMenuShell instance (a menu bar).
+ * @uiinfo: A pointer to the first element in an array of #GnomeUIInfo
+ * structures. The last element of the array should have a type of
+ * #GNOME_APP_UI_ENDOFINFO.
+ * @uibdata: A #GnomeUIInfoBuilderData instance.
+ * @accel_group: A #GtkAccelGroup.
+ * @uline_accels: %TRUE if underline accelerators will be drawn for the menu
+ * item labels.
+ * @pos: The position in the menu bar at which to start inserting items.
  *
  * Description:
  * Fills the specified menu shell with items created from the specified
- * info, inserting them from item no. pos on and using the specified
- * builder data -- this is intended for language bindings.
- * The accel group will be used as the accel group for all newly created
- * sub menus and serves as the global accel group for all menu item
- * hotkeys. If it is passed as NULL, global hotkeys will be disabled.
- * The uline_accels argument determines whether underline accelerators
- * will be featured from the menu item labels.
+ * @uiinfo, inserting them from item number @pos on and using the specified
+ * builder data (@uibdata) -- this is intended for language bindings. 
+ *
+ * The other parameters have the same meaning as in gnome_app_fill_menu().
  **/
 
 void
@@ -1487,7 +1443,9 @@ gnome_app_fill_menu_custom (GtkMenuShell       *menu_shell,
 
 				GtkWidget *menu;
 				GtkWidget *tearoff;
-
+				guint notify_id;
+				GConfClient *client;
+				
 				menu = gtk_menu_new ();
 				gtk_menu_item_set_submenu
 					(GTK_MENU_ITEM(uiinfo->widget), menu);
@@ -1497,12 +1455,25 @@ gnome_app_fill_menu_custom (GtkMenuShell       *menu_shell,
 					 uiinfo->moreinfo, orig_uibdata,
 					 accel_group, uline_accels, 0);
 
-				/* FIXME: make this runtime configurable */
 				if (gnome_gconf_get_bool ("/desktop/gnome/interface/menus_have_tearoff")) {
 					tearoff = gtk_tearoff_menu_item_new ();
 					gtk_widget_show (tearoff);
+					g_object_set_data (G_OBJECT (menu), "gnome-app-tearoff", tearoff);
 					gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), tearoff);
 				}
+
+				client = gconf_client_get_default ();
+				g_object_set_data_full(G_OBJECT(menu), gnome_app_helper_gconf_client,
+						       client, g_object_unref);
+
+				notify_id = gconf_client_notify_add (client,
+								     "/desktop/gnome/interface/menus_have_tearoff",
+								     menus_have_tearoff_changed_notify,
+								     menu, NULL, NULL);
+				g_signal_connect(menu, "destroy",
+						 G_CALLBACK(remove_notify_cb),
+						 GINT_TO_POINTER(notify_id));
+
 			}
 			pos++;
 			break;
@@ -1527,8 +1498,9 @@ gnome_app_fill_menu_custom (GtkMenuShell       *menu_shell,
 
 /**
  * gnome_app_create_menus
- * @app: Pointer to GNOME app object.
- * @uiinfo:
+ * @app: A #GnomeApp instance representing the current application.
+ * @uiinfo: The first in an array #GnomeUIInfo instances containing the menu
+ * data.
  *
  * Description:
  * Constructs a menu bar and attaches it to the specified application
@@ -1556,13 +1528,15 @@ gnome_app_create_menus (GnomeApp *app, GnomeUIInfo *uiinfo)
 
 /**
  * gnome_app_create_menus_interp
- * @app: Pointer to GNOME app object.
- * @uiinfo:
- * @relay_func:
- * @data:
- * @destroy_func:
+ * @app: A #GnomeApp instance, representing the current application.
+ * @uiinfo: The first item in an array of #GnomeUIInfo structures describing
+ * the menu bar.
+ * @relay_func: A marshaller for the signal callbacks.
+ * @data: Application specific data passed to the signal callback functions.
+ * @destroy_func: The function to call when the menu bar is destroyed.
  *
- * Description:
+ * Identical to gnome_app_create_menus(), except that extra functions and data
+ * can be passed in for finer control of the destruction and marshalling.
  **/
 
 void
@@ -1588,11 +1562,14 @@ gnome_app_create_menus_interp (GnomeApp *app, GnomeUIInfo *uiinfo,
 
 /**
  * gnome_app_create_menus_with_data
- * @app: Pointer to GNOME app object.
- * @uiinfo:
- * @user_data:
+ * @app: A #GnomeApp instance representing the current application.
+ * @uiinfo: The first in an array #GnomeUIInfo instances containing the menu
+ * data.
+ * @user_data: Application-specific data that is passed to every callback
+ * function.
  *
- * Description:
+ * Identical to gnome_app_create_menus(), except that @user_data is passed to
+ * all the callback funtions when signals are emitted.
  **/
 
 void
@@ -1661,11 +1638,14 @@ gnome_app_set_tearoff_menu_titles(GnomeApp *app, GnomeUIInfo *uiinfo,
 
 /**
  * gnome_app_create_menus_custom
- * @app: Pointer to GNOME app object.
- * @uiinfo:
- * @uibdata:
+ * @app: A #GnomeApp instance representing the current application.
+ * @uiinfo: The first in an array #GnomeUIInfo instances containing the menu
+ * data.
+ * @uibdata: An appropriate #GnomeUIBuilderData instance.
  *
- * Description:
+ * Identical to gnome_app_create_menus(), except that @uibdata is also
+ * specified for creating the signal handlers. Mostly for use by language
+ * bindings.
  **/
 
 void
@@ -1802,11 +1782,14 @@ create_radio_toolbar_items (GtkToolbar *toolbar, GnomeUIInfo *uiinfo,
 
 /**
  * gnome_app_fill_toolbar
- * @toolbar:
- * @uiinfo:
- * @accel_group:
+ * @toolbar: A #GtkToolbar instance.
+ * @uiinfo: An array of #GnomeUIInfo structures containing the items for the
+ * toolbar.
+ * @accel_group: A #GtkAccelGroup for holding the accelerator keys of the items
+ * (or %NULL).
  *
- * Description:
+ * Fills @toolbar with buttons specified in @uiinfo. If @accel_group is not
+ * %NULL, the items' accelrator keys are put into it.
  **/
 
 void
@@ -1831,12 +1814,15 @@ gnome_app_fill_toolbar (GtkToolbar *toolbar, GnomeUIInfo *uiinfo,
 
 /**
  * gnome_app_fill_toolbar_with_data
- * @toolbar:
- * @uiinfo:
- * @accel_group:
- * @user_data:
+ * @toolbar: A #GtkToolbar instance.
+ * @uiinfo: An array of #GnomeUIInfo structures containing the items for the
+ * toolbar.
+ * @accel_group: A #GtkAccelGroup for holding the accelerator keys of the items
+ * (or %NULL).
+ * @user_data: Application specific data.
  *
- * Description:
+ * The same as gnome_app_fill_toolbar(), except that the user data pointers in
+ * the signal handlers are set to @user_data.
  **/
 
 void
@@ -1861,12 +1847,15 @@ gnome_app_fill_toolbar_with_data (GtkToolbar *toolbar, GnomeUIInfo *uiinfo,
 
 /**
  * gnome_app_fill_toolbar_custom
- * @toolbar:
- * @uiinfo:
- * @uibdata:
- * @accel_group:
+ * @toolbar: A #GtkToolbar instance.
+ * @uiinfo: An array of #GnomeUIInfo structures containing the items for the
+ * toolbar.
+ * @uibdata: The #GnomeUIBuilderData data for the toolbar.
+ * @accel_group: A #GtkAccelGroup for holding the accelerator keys of the items
+ * (or %NULL).
  *
- * Description:
+ * The same as gnome_app_fill_toolbar(), except that the sepcified @uibdata
+ * instance is used. This is mostly for the benefit of language bindings.
  **/
 
 void
@@ -1917,8 +1906,8 @@ gnome_app_fill_toolbar_custom (GtkToolbar *toolbar, GnomeUIInfo *uiinfo,
 
 /**
  * gnome_app_create_toolbar
- * @app: Pointer to GNOME app object.
- * @uiinfo:
+ * @app: A #GnomeApp instance.
+ * @uiinfo: A #GnomeUIInfo array specifying the contents of the toolbar.
  *
  * Description:
  * Constructs a toolbar and attaches it to the specified application
@@ -1945,11 +1934,11 @@ gnome_app_create_toolbar (GnomeApp *app, GnomeUIInfo *uiinfo)
 
 /**
  * gnome_app_create_toolbar_interp
- * @app: Pointer to GNOME app object.
- * @uiinfo:
- * @relay_func:
- * @data:
- * @destroy_func:
+ * @app: A #GnomeApp instance.
+ * @uiinfo: A #GnomeUIInfo array specifying the contents of the toolbar.
+ * @relay_func: Argument marshalling function.
+ * @data: Application specific data to pass to signal callbacks.
+ * @destroy_func: The function to call when the toolbar is destroyed.
  *
  * Description:
  * Constructs a toolbar and attaches it to the specified application
@@ -1978,9 +1967,10 @@ gnome_app_create_toolbar_interp (GnomeApp *app, GnomeUIInfo *uiinfo,
 
 /**
  * gnome_app_create_toolbar_with_data
- * @app: Pointer to GNOME app object.
- * @uiinfo:
- * @user_data:
+ * @app: A #GnomeApp instance.
+ * @uiinfo: A #GnomeUIInfo array specifying the contents of the toolbar.
+ * @user_data: Application specific data to be sent to each signal callback
+ * function.
  *
  * Description:
  * Constructs a toolbar, sets all the user data pointers to
@@ -2008,9 +1998,10 @@ gnome_app_create_toolbar_with_data (GnomeApp *app, GnomeUIInfo *uiinfo,
 
 /**
  * gnome_app_create_toolbar_custom
- * @app: Pointer to GNOME app object.
- * @uiinfo:
- * @uibdata:
+ * @app: A #GnomeApp instance.
+ * @uiinfo: A #GnomeUIInfo array specifying the contents of the toolbar.
+ * @uibdata: A #GnomeUIBuilderData instance specifying the handlers to use for
+ * the toolbar.
  *
  * Description:
  * Constructs a toolbar and attaches it to the @app window,
@@ -2076,26 +2067,22 @@ g_strncmp_ignore_char( const gchar *first, const gchar *second, gint length, gch
 /* menu insertion/removal functions
  * <jaka.mocnik@kiss.uni-lj.si>
  *
- * the path argument should be in the form "File/.../.../Something". "" will
- * insert the item as the first one in the menubar "File/" will insert it as
- * the first one in the File menu "File/Settings" will insert it after the
- * Setting item in the File menu use of "File/<Separator>" should be obvious.
- * However this stops after the first separator. I hope this explains use of
- * the insert/remove functions well enough.
  */
 
 /**
  * gnome_app_find_menu_pos
- * @parent: Root menu shell widget containing menu items to be searched
- * @path: Specifies the target menu item by menu path
- * @pos: (output) returned item position
+ * @parent: Root menu shell widget containing menu items to be searched.
+ * @path: Specifies the target menu item by menu path.
+ * @pos: Used to hold the returned menu items' position.
  *
  * Description:
- * finds menu item described by path starting
- * in the GtkMenuShell top and returns its parent GtkMenuShell and the
- * position after this item in pos:  gtk_menu_shell_insert(p, w, pos)
- * would then insert widget w in GtkMenuShell p right after the menu item
+ * Finds a menu item described by path starting in the #GtkMenuShell top and
+ * returns its parent #GtkMenuShell and the position after this item in @pos.
+ * The meaning of @pos is that a subsequent call to gtk_menu_shell_insert(p, w,
+ * pos) would then insert widget w in GtkMenuShell p right after the menu item
  * described by path.
+ *
+ * Returns: The parent menu shell of @path.
  **/
 
 GtkWidget *
@@ -2183,12 +2170,18 @@ gnome_app_find_menu_pos (GtkWidget *parent, const gchar *path, gint *pos)
 
 /**
  * gnome_app_remove_menus
- * @app: Pointer to GNOME app object.
- * @path:
- * @items:
+ * @app: A #GnomeApp instance.
+ * @path: A path to the menu item concerned.
+ * @items: The number of items to remove.
  *
- * Description: removes num items from the existing app's menu structure
- * beginning with item described by path
+ * Description: Removes @items items from the existing @app's menu structure,
+ * beginning with item described by @path.
+ *
+ * The @path argument should be in the form "File/.../.../Something". "" will
+ * insert the item as the first one in the menubar, "File/" will insert it as
+ * the first one in the File menu, "File/Settings" will insert it after the
+ * Setting item in the File menu use of "File/<Separator>" should be obvious.
+ * However, the use of "<Seperator>" stops after the first separator.
  **/
 
 void
@@ -2235,15 +2228,15 @@ gnome_app_remove_menus(GnomeApp *app, const gchar *path, gint items)
 
 /**
  * gnome_app_remove_menu_range
- * @app: Pointer to GNOME app object.
- * @path:
- * @start:
- * @items:
+ * @app: A #GnomeApp instance.
+ * @path: A path to the menu item concerned.
+ * @start: An offset beyond the start of @path at which to begin removing.
+ * @items: The number of items to remove.
  *
  * Description:
- * Same as the gnome_app_remove_menus, except it removes the specified number
- * of @items from the existing app's menu structure begining with item described
- * by path, plus the number specified by @start - very useful for adding and
+ * Same as the gnome_app_remove_menus(), except it removes the specified number
+ * of @items from the existing @app's menu structure begining with item
+ * described by (@path plus @start). This is very useful for adding and
  * removing Recent document items in the File menu.
  **/
 
@@ -2291,13 +2284,15 @@ gnome_app_remove_menu_range (GnomeApp *app, const gchar *path, gint start, gint
 
 /**
  * gnome_app_insert_menus_custom
- * @app: Pointer to GNOME app object.
- * @path:
- * @uiinfo:
- * @uibdata:
+ * @app: A #GnomeApp instance.
+ * @path: A path to the menu item concerned.
+ * @uiinfo: A #GnomeUIInfo array describing the menus.
+ * @uibdata: A #GnomeUIBuilderData instance describing the functions to attach
+ * as the menu's callbacks.
  *
- * Description: inserts menus described by @uiinfo in existing app's menu
- * structure right after the item described by @path.
+ * Description: Inserts menus described by @uiinfo in existing @app's menu
+ * structure right after the item described by @path. The @uibdata parameter
+ * makes this, again, most useful for language bindings.
  **/
 
 void
@@ -2327,11 +2322,12 @@ gnome_app_insert_menus_custom (GnomeApp *app, const gchar *path,
 
 /**
  * gnome_app_insert_menus
- * @app: Pointer to GNOME app object.
- * @path:
- * @menuinfo:
+ * @app: A #GnomeApp instance.
+ * @path: A path to the menu item concerned.
+ * @menuinfo: A #GnomeUIInfo array describing the menus.
  *
- * Description:
+ * Insert the menus given by @menuinfo beginning at @path into the pre-existing
+ * @app.
  **/
 
 void
@@ -2351,12 +2347,13 @@ gnome_app_insert_menus (GnomeApp *app,
 
 /**
  * gnome_app_insert_menus_with_data
- * @app: Pointer to GNOME app object.
- * @path:
- * @menuinfo:
- * @data:
+ * @app: A #GnomeApp instance.
+ * @path: A path to the menu item concerned.
+ * @menuinfo: A #GnomeUIInfo array describing the menus.
+ * @data: Application specific data to send to each signal callback.
  *
- * Description:
+ * This is the same as gnome_app_insert_menus(), except that the specified
+ * @data is passed to each signal callback.
  **/
 
 void
@@ -2377,14 +2374,15 @@ gnome_app_insert_menus_with_data (GnomeApp *app, const gchar *path,
 
 /**
  * gnome_app_insert_menus_interp
- * @app: Pointer to GNOME app object.
- * @path:
- * @menuinfo:
- * @relay_func:
- * @data:
- * @destroy_func:
+ * @app: A #GnomeApp instance.
+ * @path: A path to the menu item concerned.
+ * @menuinfo: A #GnomeUIInfo array describing the menus.
+ * @relay_func: A custom marshallar for signal data.,
+ * @data: Application-specific data to send to each signal callback.
+ * @destroy_func: The function to call when the menu item is destroyed.
  *
- * Description:
+ * THe same as gnome_app_insert_menus(), except that the given functions are
+ * attached to each menu item. Mostly of use for language bindings.
  **/
 
 void
@@ -2421,7 +2419,9 @@ gnome_app_helper_gettext (const gchar *str)
 static GConfEnumStringPair toolbar_styles[] = {
         { GTK_TOOLBAR_TEXT, "text" },
         { GTK_TOOLBAR_ICONS, "icons" },
-        { GTK_TOOLBAR_BOTH, "both" }
+        { GTK_TOOLBAR_BOTH, "both" },
+        { GTK_TOOLBAR_BOTH_HORIZ, "both_horiz" },
+	{ -1, NULL }
 };
 
 static void
@@ -2456,9 +2456,10 @@ per_app_toolbar_style_changed_notify(GConfClient            *client,
                                               NULL);
 
                 if (str) {
-                        gconf_string_to_enum(toolbar_styles,
-                                             str,
-                                             (gint*)&style);
+			if (!gconf_string_to_enum (toolbar_styles,
+						   str,
+						   (gint *)&style))
+				style = GTK_TOOLBAR_BOTH;
 
                         g_free(str);
                 }
@@ -2502,9 +2503,10 @@ toolbar_style_changed_notify(GConfClient            *client,
             value &&
             value->type == GCONF_VALUE_STRING &&
             gconf_value_get_string(value) != NULL) {
-                gconf_string_to_enum(toolbar_styles,
-                                     gconf_value_get_string(value),
-                                     (gint*)&style);
+		if (!gconf_string_to_enum (toolbar_styles,
+					   gconf_value_get_string (value),
+					   (gint *)&style))
+			style = GTK_TOOLBAR_BOTH;
         }
 
         gtk_toolbar_set_style(toolbar, style);
@@ -2549,9 +2551,9 @@ static void
 create_and_popup_toolbar_menu (GdkEventButton *event)
 {
 	GtkWidget *menu;
-	GtkWidget *item, *both_item, *icons_item, *text_item, *global_item;
+	GtkWidget *item, *both_item, *both_horiz_item, *icons_item, *text_item, *global_item;
 	GSList *group;
-	char *both, *icons, *text, *global;
+	char *both, *both_horiz, *icons, *text, *global;
 	char *str, *key;
 	GtkToolbarStyle toolbar_style;
 	GConfClient *conf;
@@ -2560,9 +2562,10 @@ create_and_popup_toolbar_menu (GdkEventButton *event)
 	toolbar_style = GTK_TOOLBAR_BOTH;
 	menu = gtk_menu_new ();
 
-	both = _("Both text and icons");
-	icons = _("Icons only");
-	text = _("Text only");
+	both = _("Text Below Icons");
+	both_horiz = _("Priority Text Beside Icons");
+	icons = _("Icons Only");
+	text = _("Text Only");
 
 	both_item = gtk_radio_menu_item_new_with_label (group, both);
 	g_signal_connect (both_item, "activate",
@@ -2570,6 +2573,12 @@ create_and_popup_toolbar_menu (GdkEventButton *event)
 	group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (both_item));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), both_item);
 
+	both_horiz_item = gtk_radio_menu_item_new_with_label (group, both_horiz);
+	g_signal_connect (both_horiz_item, "activate",
+			  G_CALLBACK (style_menu_item_activated), GINT_TO_POINTER (GTK_TOOLBAR_BOTH_HORIZ));
+	group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (both_horiz_item));
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), both_horiz_item);
+
 	icons_item = gtk_radio_menu_item_new_with_label (group, icons);
 	g_signal_connect (icons_item, "activate",
 			  G_CALLBACK (style_menu_item_activated), GINT_TO_POINTER (GTK_TOOLBAR_ICONS));
@@ -2593,9 +2602,10 @@ create_and_popup_toolbar_menu (GdkEventButton *event)
 				      NULL);
 
 	if (str != NULL) {
-		gconf_string_to_enum(toolbar_styles,
-				     str,
-				     (gint*)&toolbar_style);
+		if (!gconf_string_to_enum (toolbar_styles,
+					   str,
+					   (gint*)&toolbar_style))
+			toolbar_style = GTK_TOOLBAR_BOTH;
 		g_free(str);
 	}
 
@@ -2603,6 +2613,9 @@ create_and_popup_toolbar_menu (GdkEventButton *event)
 	case GTK_TOOLBAR_BOTH:
 		str = both;
 		break;
+	case GTK_TOOLBAR_BOTH_HORIZ:
+		str = both_horiz;
+		break;
 	case GTK_TOOLBAR_ICONS:
 		str = icons;
 		break;
@@ -2633,15 +2646,19 @@ create_and_popup_toolbar_menu (GdkEventButton *event)
 		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (global_item), TRUE);
 	}
 	else {
-		gconf_string_to_enum(toolbar_styles,
-				     str,
-				     (gint*)&toolbar_style);
+		if (!gconf_string_to_enum(toolbar_styles,
+					  str,
+					  (gint*)&toolbar_style))
+			toolbar_style = GTK_TOOLBAR_BOTH;
 
 		/* We have a per-app setting, find out which one it is */
 		switch (toolbar_style) {
 		case GTK_TOOLBAR_BOTH:
 			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (both_item), TRUE);
 			break;
+		case GTK_TOOLBAR_BOTH_HORIZ:
+			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (both_horiz_item), TRUE);
+			break;
 		case GTK_TOOLBAR_ICONS:
 			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (icons_item), TRUE);
 			break;
@@ -2672,11 +2689,12 @@ button_press (GtkWidget *dock, GdkEventButton *event, gpointer user_data)
 
 /**
  * gnome_app_setup_toolbar
- * @toolbar: Pointer to #GtkToolbar widget
- * @dock_item: Pointer to a #BonoboDockItem the toolbar is inside, or NULL for none
+ * @toolbar: Pointer to a #GtkToolbar instance.
+ * @dock_item: Pointer to the #BonoboDockItem the toolbar is inside, or %NULL
+ * for none.
  *
  * Description:
- * Sets up a toolbar to use GNOME user preferences
+ * Sets up a toolbar to use GNOME user preferences.
  **/
 
 void
@@ -2743,9 +2761,10 @@ gnome_app_setup_toolbar (GtkToolbar *toolbar,
                                                       NULL);
 
                         if (str != NULL) {
-                                gconf_string_to_enum(toolbar_styles,
-                                                     str,
-                                                     (gint*)&toolbar_style);
+                                if (!gconf_string_to_enum(toolbar_styles,
+							  str,
+							  (gint*)&toolbar_style))
+					toolbar_style = GTK_TOOLBAR_BOTH;
                                 g_free(str);
                         }
                 }
