@@ -188,6 +188,14 @@ gnome_druid_page_edge_instance_init (GnomeDruidPageEdge *page)
 
 /**
  * gnome_druid_page_edge_construct:
+ * @druid_page_edge:
+ * @position:
+ * @antialiased:
+ * @title:
+ * @text:
+ * @logo:
+ * @watermark:
+ * @top_watermark:
  *
  * Description:  Useful for subclassing and binding only
  **/
@@ -327,7 +335,7 @@ gnome_druid_page_edge_new (GnomeEdgePosition position)
 	g_return_val_if_fail (position >= GNOME_EDGE_START &&
 			      position < GNOME_EDGE_LAST, NULL);
 
-	retval = gtk_type_new (GNOME_TYPE_DRUID_PAGE_EDGE);
+	retval = g_object_new (GNOME_TYPE_DRUID_PAGE_EDGE, NULL);
 
 	gnome_druid_page_edge_construct (retval,
 					 position,
@@ -359,7 +367,7 @@ gnome_druid_page_edge_new_aa (GnomeEdgePosition position)
 	g_return_val_if_fail (position >= GNOME_EDGE_START &&
 			      position < GNOME_EDGE_LAST, NULL);
 
-	retval = gtk_type_new (GNOME_TYPE_DRUID_PAGE_EDGE);
+	retval = g_object_new (GNOME_TYPE_DRUID_PAGE_EDGE, NULL);
 
 	gnome_druid_page_edge_construct (retval,
 					 position,
@@ -404,7 +412,7 @@ gnome_druid_page_edge_new_with_vals (GnomeEdgePosition position,
 	g_return_val_if_fail (position >= GNOME_EDGE_START &&
 			      position < GNOME_EDGE_LAST, NULL);
 
-	retval = gtk_type_new (GNOME_TYPE_DRUID_PAGE_EDGE);
+	retval = g_object_new (GNOME_TYPE_DRUID_PAGE_EDGE, NULL);
 
 	gnome_druid_page_edge_construct (retval,
 					 position,
@@ -541,7 +549,7 @@ gnome_druid_page_edge_set_logo (GnomeDruidPageEdge *druid_page_edge,
 /**
  * gnome_druid_page_edge_set_watermark:
  * @druid_page_edge: the #GnomeDruidPageEdge to work on
- * @top_watermark_image: The #GdkPixbuf to use as a watermark
+ * @watermark: The #GdkPixbuf to use as a watermark
  *
  * Description:  Sets a #GdkPixbuf as the watermark on the left
  * strip on the druid.  If #top_watermark_image is %NULL, it is reset
